@@ -126,7 +126,7 @@ class BaseConfiguration(object):
     def reload(self):
         self.load(self._command_line_args)
 
-    def load(self, args, accept_unexistent_config=False):
+    def load(self, args, accept_nonexistent_config=False):
         """
         Load configuration data from command line arguments and a config file.
 
@@ -138,7 +138,7 @@ class BaseConfiguration(object):
         if self.config:
             if os.path.isfile(self.config):
                 self.load_configuration_file(self.config)
-            elif not accept_unexistent_config:
+            elif not accept_nonexistent_config:
                 sys.exit("error: file not found: %s" % self.config)
         else:
             for potential_config_file in self.default_config_filenames:
@@ -146,6 +146,8 @@ class BaseConfiguration(object):
                     self.load_configuration_file(potential_config_file)
                     break
 
+        self._load_external_options()
+
         # Check that all needed options were given.
         for option in self.required_options:
             if not getattr(self, option):
@@ -156,6 +158,9 @@ class BaseConfiguration(object):
         if self.bus not in ("session", "system"):
             sys.exit("error: bus must be one of 'session' or 'system'")
 
+    def _load_external_options(self):
+        """Hook for loading options from elsewhere (e.g. for --import)."""
+
     def load_command_line(self, args):
         """Load configuration data from the given command line."""
         self._command_line_args = args
@@ -195,8 +200,7 @@ class BaseConfiguration(object):
         3. The first filename in self.default_config_filenames
         """
         # The filename we'll write to
-        filename = (self.config or self._config_filename or
-                    self.default_config_filenames[0])
+        filename = self.get_config_filename()
 
         config_parser = ConfigParser()
         # Make sure we read the old values from the config file so that we
@@ -233,7 +237,16 @@ class BaseConfiguration(object):
         return parser
 
     def get_config_filename(self):
-        return self._config_filename
+        if self.config:
+            return self.config
+        if self._config_filename:
+            return self._config_filename
+        if self.default_config_filenames:
+            for potential_config_file in self.default_config_filenames:
+                if os.access(potential_config_file, os.R_OK):
+                    return potential_config_file
+            return self.default_config_filenames[0]
+        return None
 
     def get_command_line_options(self):
         return self._command_line_options
@@ -330,11 +343,13 @@ def assert_unowned_bus_name(bus, bus_name):
         sys.exit("error: DBus name %s is owned. "
                  "Is the process already running?" % bus_name)
 
+
 _required_users = {
     "broker": "landscape",
     "monitor": "landscape",
     "manager": "root"}
 
+
 def run_landscape_service(configuration_class, service_class, args, bus_name):
     """Run a Landscape service.
 
@@ -347,7 +362,6 @@ def run_landscape_service(configuration_class, service_class, args, bus_name):
     """
     from twisted.internet.glib2reactor import install
     install()
-    from twisted.internet import reactor
 
     # Let's consider adding this:
 #     from twisted.python.log import startLoggingWithObserver, PythonLoggingObserver
@@ -368,11 +382,12 @@ def run_landscape_service(configuration_class, service_class, args, bus_name):
     assert_unowned_bus_name(get_bus(configuration.bus), bus_name)
 
     application = Application("landscape-%s" % (service_class.service_name,))
-    service_class(configuration).setServiceParent(application)
+    service = service_class(configuration)
+    service.setServiceParent(application)
 
     startApplication(application, False)
 
     if configuration.ignore_sigint:
         signal.signal(signal.SIGINT, signal.SIG_IGN)
 
-    reactor.run()
+    service.reactor.run()
