@@ -1,15 +1,21 @@
 import pycurl
 
-from landscape.lib.fetch import fetch
+from landscape.lib.fetch import fetch, fetch_async, HTTPCodeError
 from landscape.tests.helpers import LandscapeTest
 
 
 class CurlStub(object):
 
-    def __init__(self, result=None):
+    def __init__(self, result=None, http_code=200):
+        self.result = result
+        self._http_code = http_code
         self.options = {}
         self.performed = False
-        self.result = result
+
+    def getinfo(self, what):
+        if what == pycurl.HTTP_CODE:
+            return self._http_code
+        raise RuntimeError("Stub doesn't know about %d info" % what)
 
     def setopt(self, option, value):
         if self.performed:
@@ -38,6 +44,9 @@ class FetchTest(LandscapeTest):
                           {pycurl.URL: "http://example.com",
                            pycurl.FOLLOWLOCATION: True,
                            pycurl.MAXREDIRS: 5,
+                           pycurl.CONNECTTIMEOUT: 30,
+                           pycurl.TIMEOUT: 600,
+                           pycurl.NOSIGNAL: 1,
                            pycurl.WRITEFUNCTION: Any()})
 
     def test_post(self):
@@ -48,6 +57,9 @@ class FetchTest(LandscapeTest):
                           {pycurl.URL: "http://example.com",
                            pycurl.FOLLOWLOCATION: True,
                            pycurl.MAXREDIRS: 5,
+                           pycurl.CONNECTTIMEOUT: 30,
+                           pycurl.TIMEOUT: 600,
+                           pycurl.NOSIGNAL: 1,
                            pycurl.WRITEFUNCTION: Any(),
                            pycurl.POST: True})
 
@@ -60,6 +72,9 @@ class FetchTest(LandscapeTest):
                           {pycurl.URL: "http://example.com",
                            pycurl.FOLLOWLOCATION: True,
                            pycurl.MAXREDIRS: 5,
+                           pycurl.CONNECTTIMEOUT: 30,
+                           pycurl.TIMEOUT: 600,
+                           pycurl.NOSIGNAL: 1,
                            pycurl.WRITEFUNCTION: Any(),
                            pycurl.POST: True,
                            pycurl.POSTFIELDSIZE: 4,
@@ -73,6 +88,9 @@ class FetchTest(LandscapeTest):
                           {pycurl.URL: "https://example.com",
                            pycurl.FOLLOWLOCATION: True,
                            pycurl.MAXREDIRS: 5,
+                           pycurl.CONNECTTIMEOUT: 30,
+                           pycurl.TIMEOUT: 600,
+                           pycurl.NOSIGNAL: 1,
                            pycurl.WRITEFUNCTION: Any(),
                            pycurl.CAINFO: "cainfo"})
 
@@ -91,9 +109,31 @@ class FetchTest(LandscapeTest):
                           {pycurl.URL: "http://example.com",
                            pycurl.FOLLOWLOCATION: True,
                            pycurl.MAXREDIRS: 5,
+                           pycurl.CONNECTTIMEOUT: 30,
+                           pycurl.TIMEOUT: 600,
+                           pycurl.NOSIGNAL: 1,
                            pycurl.WRITEFUNCTION: Any(),
                            pycurl.HTTPHEADER: ["a: 1", "b: 2"]})
 
+    def test_non_200_result(self):
+        curl = CurlStub("result", http_code=404)
+        try:
+            fetch("http://example.com", curl=curl)
+        except HTTPCodeError, error:
+            self.assertEquals(error.http_code, 404)
+            self.assertEquals(error.body, "result")
+        else:
+            self.fail("HTTPCodeError not raised")
+
+    def test_error_str(self):
+        self.assertEquals(str(HTTPCodeError(501, "")),
+                          "Server returned HTTP code 501")
+
+    def test_error_repr(self):
+        self.assertEquals(repr(HTTPCodeError(501, "")),
+                          "<HTTPCodeError http_code=501>")
+
+
     def test_create_curl(self):
         curls = []
         def pycurl_Curl():
@@ -110,6 +150,27 @@ class FetchTest(LandscapeTest):
                               {pycurl.URL: "http://example.com",
                                pycurl.FOLLOWLOCATION: True,
                                pycurl.MAXREDIRS: 5,
+                               pycurl.CONNECTTIMEOUT: 30,
+                               pycurl.TIMEOUT: 600,
+                               pycurl.NOSIGNAL: 1,
                                pycurl.WRITEFUNCTION: Any()})
         finally:
             pycurl.Curl = Curl
+
+    def test_async_fetch(self):
+        curl = CurlStub("result")
+        d = fetch_async("http://example.com/", curl=curl)
+        def got_result(result):
+            self.assertEquals(result, "result")
+        return d.addCallback(got_result)
+
+    def test_async_fetch_with_error(self):
+        curl = CurlStub("result", http_code=501)
+        d = fetch_async("http://example.com/", curl=curl)
+        def got_error(failure):
+            self.assertEquals(failure.value.http_code, 501)
+            self.assertEquals(failure.value.body, "result")
+            return failure
+        d.addErrback(got_error)
+        self.assertFailure(d, HTTPCodeError)
+        return d
