@@ -9,8 +9,8 @@ from twisted.internet.error import ProcessDone
 from twisted.python.failure import Failure
 
 from landscape.manager.scriptexecution import (
-    ScriptExecution, ProcessTimeLimitReachedError, PROCESS_FAILED_RESULT,
-    ProcessFailedError)
+    ScriptExecutionPlugin, ProcessTimeLimitReachedError, PROCESS_FAILED_RESULT,
+    UBUNTU_PATH, get_user_info, UnknownInterpreterError)
 from landscape.manager.manager import SUCCEEDED, FAILED
 from landscape.tests.helpers import (
     LandscapeTest, LandscapeIsolatedTest, ManagerHelper,
@@ -18,13 +18,23 @@ from landscape.tests.helpers import (
 from landscape.tests.mocker import ANY, ARGS
 
 
+def get_default_environment():
+    username = os.getlogin()
+    uid, gid, home = get_user_info(username)
+    return  {
+        "PATH": UBUNTU_PATH,
+        "USER": username,
+        "HOME": home,
+        }
+
+
 class RunScriptTests(LandscapeTest):
 
     helpers = [ManagerHelper]
 
     def setUp(self):
         super(RunScriptTests, self).setUp()
-        self.plugin = ScriptExecution()
+        self.plugin = ScriptExecutionPlugin()
         self.manager.add(self.plugin)
 
     def test_basic_run(self):
@@ -42,8 +52,22 @@ class RunScriptTests(LandscapeTest):
         result.addCallback(self.assertEquals, "hi\n")
         return result
 
+    def test_other_interpreter_env(self):
+        """
+        Non-shell interpreters don't have their paths set by the shell, so we
+        need to check that other interpreters have environment variables set.
+        """
+        result = self.plugin.run_script(
+            sys.executable,
+            "import os\nprint os.environ")
+        def check_environment(results):
+            for string in get_default_environment().keys():
+                self.assertIn(string, results)
+        result.addCallback(check_environment)
+        return result
+
     def test_concurrent(self):
-        """Scripts run with the ScriptExecution plugin are run concurrently."""
+        """Scripts run with the ScriptExecutionPlugin plugin are run concurrently."""
         fifo = self.make_path()
         os.mkfifo(fifo)
         # If the first process is blocking on a fifo, and the second process
@@ -220,7 +244,7 @@ class RunScriptTests(LandscapeTest):
 
         self.assertEquals(len(factory.spawns), 1)
         spawn = factory.spawns[0]
-        self.assertEquals(spawn[3].keys(), ["LANDSCAPE_ATTACHMENTS"])
+        self.assertIn("LANDSCAPE_ATTACHMENTS", spawn[3].keys())
         attachment_dir = spawn[3]["LANDSCAPE_ATTACHMENTS"]
         self.assertEquals(stat.S_IMODE(os.stat(attachment_dir).st_mode), 0700)
         filename = os.path.join(attachment_dir, "file 1")
@@ -324,8 +348,8 @@ class RunScriptTests(LandscapeTest):
         correct permissions. Therefore os.chmod and os.chown must be called
         before data is written.
         """
-        uid = os.getuid()
-        gid = os.getgid()
+        username = os.getlogin()
+        uid, gid, home = get_user_info(username)
 
         mock_chown = self.mocker.replace("os.chown", passthrough=False)
         mock_chmod = self.mocker.replace("os.chmod", passthrough=False)
@@ -345,12 +369,10 @@ class RunScriptTests(LandscapeTest):
         # The contents are written *after* the permissions have been set up!
         script_file.write("#!/bin/sh\ncode")
         script_file.close()
-
         process_factory.spawnProcess(
-            ANY, ANY, uid=uid, gid=gid, path=ANY, env={})
-
+            ANY, ANY, uid=uid, gid=gid, path=ANY,
+            env=get_default_environment())
         self.mocker.replay()
-
         # We don't really care about the deferred that's returned, as long as
         # those things happened in the correct order.
         self.plugin.run_script("/bin/sh", "code",
@@ -378,13 +400,12 @@ class RunScriptTests(LandscapeTest):
         def cb(ignore):
             self.fail("Should not be there")
         def eb(failure):
-            failure.trap(ProcessFailedError)
+            failure.trap(UnknownInterpreterError)
             self.assertEquals(
-                failure.value.data,
-                "Unknown interpreter: '/bin/cantpossiblyexist'")
+                failure.value.interpreter,
+                "/bin/cantpossiblyexist")
         return d.addCallback(cb).addErrback(eb)
 
-
 class ScriptExecutionMessageTests(LandscapeIsolatedTest):
     helpers = [ManagerHelper]
 
@@ -402,7 +423,6 @@ class ScriptExecutionMessageTests(LandscapeIsolatedTest):
         data = open(executable, "r").read()
         self.assertEquals(data, "#!%s\n%s" % (interp, code))
 
-
     def _send_script(self, interpreter, code, operation_id=123,
                      user=pwd.getpwuid(os.getuid())[0],
                      time_limit=None):
@@ -429,7 +449,7 @@ class ScriptExecutionMessageTests(LandscapeIsolatedTest):
         mock_chown = self.mocker.replace("os.chown", passthrough=False)
         mock_chown(ARGS)
 
-        self.manager.add(ScriptExecution(process_factory=factory))
+        self.manager.add(ScriptExecutionPlugin(process_factory=factory))
 
         self.mocker.replay()
         result = self._send_script(sys.executable, "print 'hi'")
@@ -454,9 +474,8 @@ class ScriptExecutionMessageTests(LandscapeIsolatedTest):
 
     def test_user(self):
         """A user can be specified in the message."""
-        uid = os.getuid()
-        gid = os.getgid()
-        username = pwd.getpwuid(uid)[0]
+        username = os.getlogin()
+        uid, gid, home = get_user_info(username)
 
         # ignore the call to chown!
         mock_chown = self.mocker.replace("os.chown", passthrough=False)
@@ -466,15 +485,15 @@ class ScriptExecutionMessageTests(LandscapeIsolatedTest):
             protocol.childDataReceived(1, "hi!\n")
             protocol.processEnded(Failure(ProcessDone(0)))
             self._verify_script(filename, sys.executable, "print 'hi'")
-
         process_factory = self.mocker.mock()
         process_factory.spawnProcess(
-            ANY, ANY, uid=uid, gid=gid, path=ANY, env={})
+            ANY, ANY, uid=uid, gid=gid, path=ANY,
+            env=get_default_environment())
         self.mocker.call(spawn_called)
-
         self.mocker.replay()
 
-        self.manager.add(ScriptExecution(process_factory=process_factory))
+        self.manager.add(
+            ScriptExecutionPlugin(process_factory=process_factory))
 
         result = self._send_script(sys.executable, "print 'hi'", user=username)
         return result
@@ -485,7 +504,7 @@ class ScriptExecutionMessageTests(LandscapeIsolatedTest):
         operation-result should have a failed status.
         """
         factory = StubProcessFactory()
-        self.manager.add(ScriptExecution(process_factory=factory))
+        self.manager.add(ScriptExecutionPlugin(process_factory=factory))
 
         # ignore the call to chown!
         mock_chown = self.mocker.replace("os.chown", passthrough=False)
@@ -517,7 +536,7 @@ class ScriptExecutionMessageTests(LandscapeIsolatedTest):
         Messages which try to run a script as a user that is not allowed should
         be rejected.
         """
-        self.manager.add(ScriptExecution())
+        self.manager.add(ScriptExecutionPlugin())
         self.manager.config.script_users = "landscape, nobody"
         result = self._send_script(sys.executable, "bar", user="whatever")
         def got_result(r):
@@ -532,9 +551,8 @@ class ScriptExecutionMessageTests(LandscapeIsolatedTest):
 
     def test_urgent_response(self):
         """Responses to script execution messages are urgent."""
-        uid = os.getuid()
-        gid = os.getgid()
-        username = pwd.getpwuid(uid)[0]
+        username = os.getlogin()
+        uid, gid, home = get_user_info(username)
 
         # ignore the call to chown!
         mock_chown = self.mocker.replace("os.chown", passthrough=False)
@@ -544,15 +562,15 @@ class ScriptExecutionMessageTests(LandscapeIsolatedTest):
             protocol.childDataReceived(1, "hi!\n")
             protocol.processEnded(Failure(ProcessDone(0)))
             self._verify_script(filename, sys.executable, "print 'hi'")
-
         process_factory = self.mocker.mock()
         process_factory.spawnProcess(
-            ANY, ANY, uid=uid, gid=gid, path=ANY, env={})
+            ANY, ANY, uid=uid, gid=gid, path=ANY,
+            env=get_default_environment())
         self.mocker.call(spawn_called)
 
         self.mocker.replay()
 
-        self.manager.add(ScriptExecution(process_factory=process_factory))
+        self.manager.add(ScriptExecutionPlugin(process_factory=process_factory))
 
         def got_result(r):
             self.assertTrue(self.broker_service.exchanger.is_urgent())
@@ -573,7 +591,7 @@ class ScriptExecutionMessageTests(LandscapeIsolatedTest):
         sent (assuming operation-id *is* successfully parsed).
         """
         self.log_helper.ignore_errors(KeyError)
-        self.manager.add(ScriptExecution())
+        self.manager.add(ScriptExecutionPlugin())
 
         self.manager.dispatch_message(
             {"type": "execute-script", "operation-id": 444})
@@ -600,7 +618,7 @@ class ScriptExecutionMessageTests(LandscapeIsolatedTest):
         self.mocker.count(0, None)
         self.mocker.replay()
 
-        self.manager.add(ScriptExecution())
+        self.manager.add(ScriptExecutionPlugin())
         result = self._send_script("/bin/sh", "echo hi; exit 1")
 
         def got_result(ignored):
@@ -625,7 +643,7 @@ class ScriptExecutionMessageTests(LandscapeIsolatedTest):
         mock_chown = self.mocker.replace("os.chown", passthrough=False)
         mock_chown(ARGS)
 
-        self.manager.add(ScriptExecution(process_factory=factory))
+        self.manager.add(ScriptExecutionPlugin(process_factory=factory))
 
         self.mocker.replay()
         result = self._send_script(sys.executable, "print 'hi'")
