@@ -144,6 +144,22 @@ REGISTER = Message(
     # hostname wasn't around in old versions
     optional=["registration_password", "hostname"])
 
+REGISTER_CLOUD_VM = Message(
+    "register-cloud-vm",
+    {"hostname": utf8,
+     "otp":  Any(String(), Constant(None)),
+     "instance_key": Unicode(),
+     "account_name": Any(utf8, Constant(None)),
+     "registration_password": Any(utf8, Constant(None)),
+     "reservation_key": Unicode(),
+     "public_hostname": Unicode(),
+     "local_hostname": Unicode(),
+     "kernel_key": Unicode(),
+     "ramdisk_key": Unicode(),
+     "launch_index": Int(),
+     "image_key": Unicode(),
+     })
+
 TEMPERATURE = Message("temperature", {
     "thermal-zone": utf8,
     "temperatures": List(Tuple(Int(), Float())),
@@ -313,7 +329,8 @@ for schema in [ACTIVE_PROCESS_INFO, COMPUTER_UPTIME, CLIENT_UPTIME,
                OPERATION_RESULT, COMPUTER_INFO, DISTRIBUTION_INFO,
                HARDWARE_INVENTORY, LOAD_AVERAGE, MEMORY_INFO,
                RESYNCHRONIZE, MOUNT_ACTIVITY, MOUNT_INFO, FREE_SPACE,
-               REGISTER, TEMPERATURE, PROCESSOR_INFO, USERS, PACKAGES,
+               REGISTER, REGISTER_CLOUD_VM, TEMPERATURE, PROCESSOR_INFO,
+               USERS, PACKAGES,
                CHANGE_PACKAGES_RESULT, UNKNOWN_PACKAGE_HASHES,
                ADD_PACKAGES, TEXT_MESSAGE, TEST, CUSTOM_GRAPH]:
     message_schemas[schema.type] = schema
